import logging
import time
import boto3

from cast_ai.se.models.cloud_confs import AwsConfig
from cast_ai.se.models.execution_status import ExecutionStatus
from cast_ai.se.contollers.cloud_controller_svc import CloudController


class EKSController(CloudController):
    def __init__(self, aws_conf: AwsConfig):
        try:
            self._asg = aws_conf.default_node_group
            self._logger = logging.getLogger(__name__)
            self._client = boto3.client(
                'autoscaling',
                region_name=aws_conf.region,
                aws_access_key_id=aws_conf.access_key,
                aws_secret_access_key=aws_conf.access_secret_key
            )
        except Exception as e:
            self._logger.critical(f"An error occurred during EKSController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during EKSController initialization: {str(e)}")

    def scale_default_ng(self, node_count: int) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Scaling EKS Auto Scaling Group to {node_count} ]")

        try:
            for _ in range(5):
                response = self._client.update_auto_scaling_group(
                    AutoScalingGroupName=self._asg,
                    MinSize=node_count,
                    MaxSize=node_count + 1,  # You can adjust the max size as needed
                    DesiredCapacity=node_count,
                )
                if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
                    if not self._is_ng_scaled(node_count):
                        self._logger.warning(f"Default ng=({self._asg}) has still not changed going to retry")
                        time.sleep(1)
                        continue
                    else:
                        self._logger.debug(f"Successfully updated AWS Auto Scaling Group [{self._asg}] : [{str()}]")
                        self._logger.info('Successfully updated AWS Auto Scaling Group')
                        return ExecutionStatus()
                else:
                    self._logger.error(f"Scaling default ng result = [{response}]")
                    return ExecutionStatus(f'HTTP {response["ResponseMetadata"]["HTTPStatusCode"]}')
        except Exception as e:
            self._logger.exception(f"Error trying to set Demo Nodes capacity: {str(e)}")
            raise RuntimeError(f"Error trying to set Demo Nodes capacity: {str(e)}")

    def _is_ng_scaled(self, node_count: int) -> bool:
        validate_ng_response = self._client.describe_auto_scaling_groups(AutoScalingGroupNames=[self._asg])
        self._logger.debug(f"Validating ASG={validate_ng_response['AutoScalingGroups'][0]}")
        return (validate_ng_response["AutoScalingGroups"][0]["DesiredCapacity"] == node_count and
                validate_ng_response["AutoScalingGroups"][0]["MinSize"] == node_count)



import logging

import requests
from typing import Dict, Any

# Set up logger
logger = logging.getLogger(__name__)


class CustomHTTPError(Exception):
    def __init__(self, message="HTTP Error", response=None):
        super().__init__(message)
        self.response = response


def handle_request(url: str, headers: dict, method: str, json: Dict[str, Any] = None) -> requests.Response:
    logger.debug(f"Issuing a {method} request to {url}")
    response = None
    try:
        if method == 'GET':
            response = requests.get(url, headers=headers)
        elif method == 'PUT':
            response = requests.put(url, headers=headers, json=json)
        elif method == 'DELETE':
            response = requests.delete(url, headers=headers)
        else:
            logger.error(f"Unsupported HTTP method: {method}")
            raise ValueError(f"Unsupported HTTP method: {method}")
        response.raise_for_status()
        return response
    except requests.exceptions.HTTPError as errh:
        logger.error(f"HTTP Error: {errh}")
        raise CustomHTTPError(f"HTTP Error: {errh}")
    except requests.exceptions.ConnectionError as errc:
        logger.error(f"Error Connecting: {errc}")
        raise CustomHTTPError(f"Error Connecting: {errc}")
    except requests.exceptions.Timeout as errt:
        logger.error(f"Timeout Error: {errt}")
        raise CustomHTTPError(f"Timeout Error: {errt}")
    except requests.exceptions.RequestException as err:
        logger.error(f"Something went wrong: {err}")
        raise CustomHTTPError(f"Something went wrong: {err}")


def get_api_key_headers(cast_api_key: str) -> Dict[str, str]:
    api_key = cast_api_key

    if not api_key:

        raise ValueError("CAST_API_KEY was not set in secrets.env file or file was not found")
    headers = {
        "accept": "application/json",
        "Authorization": f"Bearer {api_key}",
        "X-API-Key": api_key
    }
    return headers



from setuptools import setup, find_packages
import os

with open(os.path.join('version.txt')) as version_file:
    version_from_file = version_file.read().strip()

with open('requirements.txt') as f_required:
    required = f_required.read().splitlines()

with open('test_requirements.txt') as f_tests:
    required_for_tests = f_tests.read().splitlines()

setup(
    name="cast_ai-se-tools",
    author="Dan Amzulescu",
    author_email="dan@cast.ai",
    description="Provides tools for SE projects (e.g. Cloud controllers).",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    test_suite='nose.collector',
    test_requires=required_for_tests,
    package_data={'': ['*.txt']},
    install_requires=required,
    version=version_from_file,
    include_package_data=True,
    keywords="cast.ai k8s eks aks gke",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Customer Service",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python"
    ],
    requires=[],
    python_requires=">=3.6"
)



import logging


class AwsConfig:
    def __init__(self, aws_conf: dict):
        self._logger = self._logger = logging.getLogger(__name__)
        try:
            self.region = aws_conf["AWS_REGION"]
            self.default_node_group = aws_conf["AWS_DEFAULT_NODE_GROUP"]
            self.access_key = aws_conf["AWS_ACCESS_KEY"]
            self.access_secret_key = aws_conf["AWS_ACCESS_SECRET_KEY"]
        except Exception as e:
            self._logger.critical(f"Was not able to initialize aws config:{str(e)}")
            raise RuntimeError(f"Was not able to initialize aws config:{str(e)}")
